@@ -1,14 +1,15 @@
-pub use io::{Readable, Writable, VarInt, VarLong};
-
 pub mod packets;
 pub mod io;
+pub mod error;
+
+pub use io::*;
+pub use error::*;
 
 #[cfg(test)]
 mod tests {
-    use std::io::Cursor;
+    use std::io::{Cursor};
 
-    use crate::*;
-    use crate::io::VarInt;
+    use crate::{Writable, Readable, packet_data, packets, VarInt};
 
     #[test]
     fn it_works() {
@@ -19,7 +20,7 @@ mod tests {
             }
 
             struct TestStruct (->) {
-                Name: String
+                name: String
             }
         }
 
@@ -27,21 +28,22 @@ mod tests {
         packets! {
             BiPackets (<->) {
                 TestA (0x01) {
-                    b: VarInt
+                    b: VarInt,
+                    a: Vec<u8>,
                 }
                 TestB (0x02) {}
             }
         }
 
 
-        let mut p = TestA {
-           b: VarInt(4294967295)
+        let mut p = BiPackets::TestA {
+            b: VarInt(4294967295),
+            a: vec![1,2,5]
         };
         println!("{:?}", p);
 
 
         let mut o = Vec::new();
-        (BiPackets::TestA(p)).write(&mut o);
         match p.write(&mut o) {
             Err(_) => println!("Failed to encode"),
             Ok(_) => {
@@ -50,9 +52,10 @@ mod tests {
                 match BiPackets::read(&mut s) {
                     Err(e) => println!("{:?}",e),
                     Ok(p) => {
+                        println!("{:?}",p);
                         match p {
-                            BiPackets::TestA(p) => {
-                                print!("{:?}", p)
+                            BiPackets::TestA {b, a} => {
+                                print!("{:?} {:?}", b, a)
                             }
                             _ => {}
                         }
